<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\DailyQualityBagInspectionData;
use Modules\Qc\Entities\DailyQualityBagInspectionDataHeader;

class DailyQualityBagInspectionDataHeaderController extends Controller
{
    public function allRecords()
    {
        try {
            $query = "SELECT
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            qc_daily_quality_bag_inspection_data_headers.*,
            hr_employee_sites.site,
            hr_att_shifts.shift_name
        FROM
            st_products
            INNER JOIN
            pp_customer_order_plans_data
            INNER JOIN
            qc_daily_quality_bag_inspection_data_headers
            ON
                st_products.product_primary_id = qc_daily_quality_bag_inspection_data_headers.product_primary_id AND
                pp_customer_order_plans_data.customer_order_plan_data_id = qc_daily_quality_bag_inspection_data_headers.customer_order_plan_data_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_daily_quality_bag_inspection_data_headers.factory = hr_employee_sites.site_id
            INNER JOIN
            hr_att_shifts
            ON
                qc_daily_quality_bag_inspection_data_headers.shift = hr_att_shifts.shift_id
        GROUP BY
            qc_daily_quality_bag_inspection_data_headers.dq_bag_inspection_id";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "dq_bag_inspection_id" => $record->dq_bag_inspection_id,
                    "reference_no" => $record->reference_no,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "shift" => $record->shift_name,
                ]);
            }
            $responseBody = $this->responseBody(true, "DailyQualityBagInspectionDataHeaderController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyQualityBagInspectionDataHeaderController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function deleteRecord($id)
    {
        try {
            if ($this->delete($id)) {
                $rec_head = DailyQualityBagInspectionDataHeader::where('dq_bag_inspection_id', '=', $id)->get();
                foreach ($rec_head as $data) {
                    $data->delete();
                }
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }

    public function delete($id)
    {
        try {
            $rec_data = DailyQualityBagInspectionData::where('dq_bag_inspection_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }

    public function bagInspectionReport($id)
    {
        try {
            $query1 = "SELECT
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            hr_employee_sites.site,
            hr_att_shifts.shift_name,
            qc_daily_quality_bag_inspection_data_headers.*
        FROM
            st_products
            INNER JOIN
            pp_customer_order_plans_data
            INNER JOIN
            qc_daily_quality_bag_inspection_data_headers
            ON
                st_products.product_primary_id = qc_daily_quality_bag_inspection_data_headers.product_primary_id AND
                pp_customer_order_plans_data.customer_order_plan_data_id = qc_daily_quality_bag_inspection_data_headers.customer_order_plan_data_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_daily_quality_bag_inspection_data_headers.factory = hr_employee_sites.site_id
            INNER JOIN
            hr_att_shifts
            ON
                qc_daily_quality_bag_inspection_data_headers.shift = hr_att_shifts.shift_id
        WHERE
            qc_daily_quality_bag_inspection_data_headers.dq_bag_inspection_id = {$id}";

            $records = DB::select($query1);

            $header = [
                "reference_no" => "",
                "product" => "",
                "factory_order_number" => "",
                "factory" => "",
                "shift" => "",
                "size_of_product" => "",
                "date" => "",
            ];


            if ($records) {
                $header["reference_no"] = $records[0]->reference_no;
                $header["product"] = $records[0]->product_name;
                $header["factory_order_number"] = $records[0]->manufacturing_order_no;
                $header["date"] = $records[0]->date;
                $header["factory"] = $records[0]->site;
                $header["shift"] = $records[0]->shift_name;
                $header["size_of_product"] = $records[0]->size_of_product;
            }

            $query2 = "SELECT
            qc_daily_quality_bag_inspection_data.*
        FROM
            qc_daily_quality_bag_inspection_data
        WHERE
            qc_daily_quality_bag_inspection_data.dq_bag_inspection_id = {$id}";

            $data = DB::select($query2);



            $responseBody = $this->responseBody(true, "DailyQualityBagInspectionDataHeaderController", "bagInspectionReport", ["header" => $header, "body" => ["data" => $data]]);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyQualityBagInspectionDataHeaderController", "bagInspectionReport", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
