<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\OutsideFactoryVisitRecord;
use Modules\Qc\Entities\OutsideFactoryVisitRecordAttachment;

class OutsideFactoryVisitRecordController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('factory_visit'), $filename);

            $factory_visit_attachment = new OutsideFactoryVisitRecordAttachment();
            $factory_visit_attachment->outside_factory_visit_id = 0;
            $factory_visit_attachment->path = $filename;
            $factory_visit_attachment->token = $token;
            $factory_visit_attachment->save();

            $responseBody = $this->responseBody(true, "OutsideFactoryVisitRecordController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "OutsideFactoryVisitRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_outside_factory_visit_records.*,
            qc_outside_factory_visit_record_attachments.path,
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            hr_employee_sites.site
        FROM
            qc_outside_factory_visit_record_attachments
            INNER JOIN
            qc_outside_factory_visit_records
            ON
                qc_outside_factory_visit_record_attachments.outside_factory_visit_id = qc_outside_factory_visit_records.outside_factory_visit_id
            INNER JOIN
            pp_customer_order_plans_data
            ON
                qc_outside_factory_visit_records.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            INNER JOIN
            st_products
            ON
                qc_outside_factory_visit_records.product_primary_id = st_products.product_primary_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_outside_factory_visit_records.factory = hr_employee_sites.site_id
                GROUP BY
            qc_outside_factory_visit_records.outside_factory_visit_id
            ";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "outside_factory_visit_id" => $record->outside_factory_visit_id,
                    "reference_no" => $record->reference_no,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "OutsideFactoryVisitRecordController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "OutsideFactoryVisitRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveRecord(Request $request)
    {
        try {
            $visit = new OutsideFactoryVisitRecord();
            $visit->user_id = Auth::user()->id;
            $visit->reference_no = $request->get('reference_no');
            $visit->date = $request->get('date');
            $visit->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $visit->product_primary_id = $request->get('product_primary_id');
            $visit->factory = $request->get('factory');
            $visit->description = $request->get('description');
            if ($visit->save()) {
                $token = $request->get('attachment_token');
                $attachment = OutsideFactoryVisitRecordAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->outside_factory_visit_id = $visit->outside_factory_visit_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "OutsideFactoryVisitRecordController", "saveRecord", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "OutsideFactoryVisitRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $visit = OutsideFactoryVisitRecord::find($id);
            $visit->user_id = Auth::user()->id;
            $visit->reference_no = $request->get('reference_no');
            $visit->date = $request->get('date');
            $visit->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $visit->product_primary_id = $request->get('product_primary_id');
            $visit->factory = $request->get('factory');
            $visit->description = $request->get('description');
            if ($visit->update()) {
                $token = $request->get('attachment_token');
                $attachments = OutsideFactoryVisitRecordAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = OutsideFactoryVisitRecordAttachment::where('token', '=', $token)->first();
                        $attachment->outside_factory_visit_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "OutsideFactoryVisitRecordController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "OutsideFactoryVisitRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = OutsideFactoryVisitRecordAttachment::where('outside_factory_visit_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = OutsideFactoryVisitRecord::where('outside_factory_visit_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = OutsideFactoryVisitRecordAttachment::where('outside_factory_visit_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "OutsideFactoryVisitRecordController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "OutsideFactoryVisitRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getData($id)
    {
        try {
            $query = "SELECT
            qc_outside_factory_visit_records.*,
            qc_outside_factory_visit_record_attachments.token
        FROM
            qc_outside_factory_visit_record_attachments
            INNER JOIN
            qc_outside_factory_visit_records
            ON
                qc_outside_factory_visit_record_attachments.outside_factory_visit_id = qc_outside_factory_visit_records.outside_factory_visit_id
        WHERE
            qc_outside_factory_visit_records.outside_factory_visit_id = {$id}";

            $visit =  DB::select($query);

            $responseBody = $this->responseBody(true, "OutsideFactoryVisitRecordController", "getData", $visit[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "OutsideFactoryVisitRecordController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
